include ht48e30.inc

;***********************************************************************************
; Based on 8051 C code from MST, (c) Sensirion AG
; only main functions have been tested
;***********************************************************************************/
; SLO HAN SIDO PROBADAS LAS FUNCIONES PRINCIPALES (las descriptas en CAN-047)


; DAT must be LSB, otherwise, code must be modified
DAT		EQU	PB.0
DATPORT EQU PB
DATT	EQU PBC.0
SCK		EQU PB.1


SHT_TMOUT equ	2000

;                            addr  command  r/w
STATUS_REG_W EQU	006h   ; 000   0011    0
STATUS_REG_R EQU	007h   ; 000   0011    1
MEASURE_TEMP EQU	003h   ; 000   0001    1
MEASURE_HUMI EQU	005h   ; 000   0010    1
SHT_RESET    EQU	01Eh   ; 000   1111    0

shtvars	.section 'data'
datas	db ?
bitcntr	db ?
value	db	2 dup(?)
chksum	db ?
flags	db ?

ACK EQU	flags.0


		public value,chksum


shtcode	.section 'CODE'
; Send  a byte
; Call with data in A
; Returns status in C flag
; C -> Error
s_write_byte:
		mov datas,a				; Store data
		mov a,8					; 8 bits
		mov bitcntr,a
swbl1:	rlc datas				; C = MSB (shift bit for output)
		clr SCK					; CLK = 0
		nop
		clr DAT					; DATA = 0 (open-collector)
		nop
		set DATT				; pull-up
		snz C					; if C, DATA = pull-up
		clr DATT				; otherwise DATA = 0
		nop
		set SCK					; CLK = 1
		sdz bitcntr
		jmp swbl1
		clr SCK					; CLK = 0
		nop
		set DATT				; pull-up
		nop
		set SCK					; CLK = 1 (9th clk)
		rrca DATPORT			; check ack (DATA will be pulled down by SHT11)
		clr SCK					; CLK = 0
		ret

; Receive a byte
; Returns data in A
s_read_byte:
		clr datas				; DATA = 0
		mov a,8					; 8 bits
		mov bitcntr,a
srbl1:	set SCK					; CLK = 1
		nop
		rrca DATPORT			; read data in C
		rlc datas				; and into register	(MSB first)	
		clr SCK					; CLK = 0
		sdz bitcntr
		jmp srbl1
		clr DAT					; DATA = 0 (open-collector)
		nop
		sz ACK					; if need to ACK
		clr DATT				; DATA=0 (ACK)
		nop
		set SCK					; CLK = 1 (9th clk)
		nop
		clr SCK					; CLK = 0
		nop
		set DATT				; pull-up
		mov a,datas
		ret


; communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
;       _____________________________________________________         ________
; DATA:                                                      |_______|
;          _    _    _    _    _    _    _    _    _        ___     ___
; SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
; Call first time, or when you sense errors
s_connectionreset:
		clr SCK					; CLK = 0
		mov a,9					; 9 SCK cycles
		mov bitcntr,a
scrl1:	set SCK					; CLK = 1 
		nop
		clr SCK					; CLK = 0
		sdz bitcntr
		jmp scrl1
  				                 ; follow through transmission start
;generates a transmission start
;       _____         ________
; DATA:      |_______|
;           ___     ___
; SCK : ___|   |___|   |______
s_transstart:
		clr SCK					; CLK = 0
		nop
		set SCK					; CLK = 1
		nop
		clr DAT					; DATA = 0 (open-collector)
		nop
		clr DATT				; DATA=0
		nop
		clr SCK					; CLK = 0
		nop
		set SCK					; CLK = 1
		nop
		set DATT				; pull-up
		nop
		clr SCK					; CLK = 0
		ret

s_softreset:
; resets the sensor by a softreset
; Returns write_byte error
		call s_connectionreset	; reset communication
		mov a,SHT_RESET
   		jmp s_write_byte		; send RESET-command to sensor

; reads the status register with checksum (8-bit)
; Returns write_byte error
s_read_statusreg:
		call s_transstart		;  transmission start
		mov a,STATUS_REG_R
		call s_write_byte		; send command to sensor
		sz C					; return on error
		ret
		set ACK					; ACK
		call s_read_byte		; read status register (8-bit)
		mov value,a
		clr ACK					; no ACK
  		call s_read_byte		; read checksum (8-bit)
		mov chksum,a
		ret

; writes the status register with checksum (8-bit)
; Returns write_byte error
s_write_statusreg:
		call s_transstart		; transmission start
		mov a,STATUS_REG_W
		call s_write_byte		; send command to sensor
		sz C					; return on error
		ret
		mov a,value
		jmp s_write_byte		; send value of status register

; makes a measurement (temperature) with checksum
s_measure_t:
		call s_transstart		; transmission start
		mov a,MEASURE_TEMP
		jmp s_write_byte		; send command to sensor

; makes a measurement (humidity) with checksum
s_measure_rh:
		call s_transstart		; transmission start
		mov a,MEASURE_HUMI
		jmp s_write_byte		; send command to sensor

; Call until it returns C reset or you get tired, whatever comes first
s_measure_wait:
		rrca DATPORT			; read data in C
		ret

; Get measurement results out of sensor
s_measure_get:
		set ACK					; ACK
		call s_read_byte		; read the first byte (MSB)
		mov value[1],a
		call s_read_byte		; read the second byte (LSB)
		mov value,a
		clr ACK					; no ACK
  		call s_read_byte		; read checksum (8-bit)
		mov chksum,a
		ret

		public s_connectionreset,s_measure_t,s_measure_rh,s_measure_wait
		public s_measure_get,s_read_statusreg,s_write_statusreg,s_softreset

		END
