include ht48e30.inc

heater	EQU	PC.0
fan		EQU	PC.1
alarm	EQU	PC.2

guardtime	EQU	3
stimeout	EQU	500
ssleep		EQU	1

clockrate equ 4000000         ;define clock rate here
toconst	equ	(1+(stimeout*clockrate)/(4000*256*20))
slconst	equ	(1+(ssleep*clockrate)/(4*256*20))
guardconst	equ	(1+(guardtime*clockrate)/(4000*256*8))



vars	.section 'DATA'
delay	db 2 dup(?)
shtstatus	db ?
temp	db 2 dup(?)
humi	db 2 dup(?)
temphi	db 2 dup(?)
templo	db 2 dup(?)
temphhi	db 2 dup(?)
shttime	db 3 dup(?)

SHT_START	EQU	0
SHT_MEAS_T	EQU	1
SHT_MEAS_RH	EQU	2
SHT_SLEEP	EQU	3

		EXTERN 	trw_sendpacket:near,trw_getpacket:near,trw_setrx:near,trw_setch:near,trw_init:near,trw_address:near
		EXTERN	s_connectionreset:near,s_measure_t:near,s_measure_rh:near,s_measure_wait:near
		EXTERN	s_measure_get:near
		EXTERN	messge:byte,value:byte

reset	.section at 0 'CODE'
		ORG 0
		jmp start

		ORG 00Ch
start:  clr	PC			; 
		clr PB 
        set PAC					; PA0-7 = inputs
        mov a,0F1h				; PB1-3 = outputs, PB0,4-7 = inputs
		mov PBC,a
	    clr PCC					; PC0-7 = outputs
		set PGC					; PG0 = input

		call TXdelay			; Must wait at poweron before
		call trw_init			; init TRW module (returns A=0)

		mov a,0					; select master
		call trw_address		; Set master address (to answer back)
		clr shtstatus
		clr shttime


main:	call trw_setrx			; Enable rx
mloop:	call trw_getpacket		; Get msg
		snz ACC.0				; A!=0, only bit0 checked (trw_getpacket returns 00 or FF)
		jmp nomsg				; no, skip
		call TXdelay 			; give the master some time to enable Rx (Rabbit CAN-045 waits 3ms after Tx)
		mov a,messge			; update limits
		mov templo,a
		mov a,messge[1]			; templo: lower limit
		mov templo[1],a
		mov a,messge[2]			; temphi: high limit
		mov temphi,a
		mov a,messge[3]
		mov temphi[1],a
		mov a,messge[4]			; temphhi: highest limit (alarm)
		mov temphhi,a
		mov a,messge[5]
		mov temphhi[1],a
		mov a,shtstatus		; Return sensor status
		mov messge,a
		mov a,temp				; last temperature measurement
		mov messge[1],a
		mov a,temp[1]
		mov messge[2],a
		mov a,humi				; last humidity measurement
		mov messge[3],a
		mov a,humi[1]
		mov messge[4],a
		clr ACC
		sz heater				; outputs
		set ACC.3				; convert to the bit positions the Rabbit expects
		sz fan
		set ACC.4
		sz alarm
		set ACC.5
		mov messge[5],a
		call trw_sendpacket		; and answer back
		call TXdelay 			; must wait >3ms so Tx goes out
		jmp main

nomsg:	mov a,shtstatus
		addm a,PCL
		jmp shtrst
		jmp shtmeat
		jmp shtmeah
		jmp shtslp

shtrst:
		clr shtstatus
		call s_connectionreset	; reset sensor
shtstrt:
		call s_measure_t		; measure temperature
		snz C					; reset sensor on error
		inc shtstatus			; wait for result
		mov a,toconst			; init delay constant
		mov shttime[1],a
		jmp mloop
shtmeat:
		call chktmout
		sz C					; reset on timeout
		jmp shtrst
		call s_measure_wait
		sz C					; return if need to wait
		jmp mloop
		call s_measure_get		; get results
		mov a,value
		mov temp,a
		mov a,value[1]
		mov temp[1],a
		call s_measure_rh		; measure humidity
		sz C					; reset on error
		jmp shtrst
		inc shtstatus			; next
		mov a,toconst			; init delay constant
		mov shttime[1],a
		jmp mloop
shtmeah:
		 call chktmout
		sz C					; reset on timeout
		jmp shtrst
		call s_measure_wait
		sz C					; return if need to wait
		jmp mloop
		call s_measure_get		; get results
		mov a,value
		mov humi,a
		mov a,value[1]
		mov humi[1],a
chkt:	mov a,temp[1]			; check limits: t < templo ?
		sub a,templo[1]
		snz C
		jmp tunder
		sz ACC
		jmp chkt1
		mov a,temp
		sub a,templo
		snz C
		jmp tunder
chkt1:	clr heater				; t > templo, heater off
		mov a,temphi[1]			; check limits: t > temphi ?
		sub a,temp[1]
		snz C
		jmp tover
		sz ACC
		jmp tok
		mov a,temphi
		sub a,temp
		sz C
		jmp tok
tover:	set fan					; t > temphi, fan on
		mov a,temphhi[1]		; check limits: t > temphhi ?
		sub a,temp[1]
		snz C
		jmp talarm
		sz ACC
		jmp chkdone
		mov a,temphhi
		sub a,temp
		sz C
		jmp chkdone
talarm:	set alarm				; t > temphhi, alarm on
		jmp chkdone
tunder:	set heater				; t < templo, heater on
tok:	clr fan					; fan off
		clr alarm				; alarm off
chkdone: inc shtstatus			; next (sleep for some time)
		mov a,slconst			; init delay constant
		mov shttime[1],a
		jmp mloop
shtslp:
		call chktmout
		snz C					; return if need to wait
		jmp mloop
		clr shtstatus
		jmp shtstrt				; restart measurement

chktmout:
		clr C
		sdz shttime
		ret
		sdz shttime[1]
		ret
		set C
		ret



TXdelay:
		clr delay
		mov a,guardconst
		mov delay[1],a
l1:		jmp l1a
l1a:	jmp l1b
l1b:	nop
		sdz delay
		jmp l1
		sdz delay[1]
		jmp l1
		ret

		end
